# --include-- ../build/perl-invocation
# snap-clean - remove detritus of failed snap-snapshot operations
#
# Copyright (C) 2002 Kyle E. Jones
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program, in a file named COPYING.

# --include-- common.pl
# --include-- common-client.pl
# --include-- ../build/system.pl

sub
usage
{
   fatal
"usage: $progname [options]
options are:
   --verbose
   --registration-file=FILE
   --ssh-options=OPS
";
}

my $ssh_options;

usage unless
   GetOptions(
	'verbose|debug+' => \$debug,
	'ssh-options=s' => \$ssh_options,
	'registration-file=s' => \$registration_file,
   );

my $r = read_registrations;

my $key;

my $hostname = `$hostname_program`;
chomp $hostname;

# The cleaning process:
#
# Loop through all the entries in the registration file, issuing
# a CLEAN command for each repository ID.  If consecutive entries
# refer to the same host or to the local filesystem we reuse the
# existing connection to the backend.

my $talking_to = '';
my $connection_open = 0;
my $line;
my $errors = 0;
foreach $key (keys %{ $r }) {
   my $id = $r->{$key}{id};
   my $user = $r->{$key}{user};
   my $remotehost = $r->{$key}{remotehost};
   my $repositories = $r->{$key}{repositories};

   if ($connection_open && $talking_to eq $remotehost) {
      # entry shares the same remote host as laste entry or
      # both refer to the local filesystem, so reuse connection
      ;
   } else {
      # can't reuse connection; close it and open a new one
      if ($connection_open) {
         send_command "QUIT";
         read_response;
         close FROMBACKEND;
         close TOBACKEND;
         $connection_open = 0;
      }
      if ($remotehost ne '') {
	 open2(\*FROMBACKEND, \*TOBACKEND,
	       "$ssh_program $ssh_options -x -l $user $remotehost $snap_backend_program");
      } else {
	 open2(\*FROMBACKEND, \*TOBACKEND, "$snap_backend_program");
      }
      $connection_open = 1;
      $talking_to = $remotehost;

      autoflush TOBACKEND 1;

      $line = read_response;
      fatal "no greeting from remote server" if $line eq '';
      fatal "remote server greeted us with ",
         safety_decode($line) if $line !~ /^OK/;
   }

   send_command "CLEAN", $repositories, $id;
   $line = read_response;
   if ($line !~ /^OK/) {
      diag "CLEAN failed, remote server said ", safety_decode($line);
      $errors++;
   }
}

if ($connection_open) {
   send_command "QUIT";
   read_response;
}

exit $errors ? 1 : 0;
