# --include-- ../build/perl-invocation
# snap-unregister - remove directory registration and its snapshots
#
# Copyright (C) 2002 Kyle E. Jones
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 1, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program, in a file named COPYING.

# --include-- common.pl
# --include-- common-client.pl
# --include-- ../build/system.pl

sub
usage
{
   fatal
"usage: $progname [options] directory
options are:
   --verbose
   --registration-file=FILE
   --ssh-options=OPTIONS
";
}

my $ssh_options;

usage unless
   GetOptions(
	'verbose|debug+' => \$debug,
	'registration-file=s' => \$registration_file,
	'ssh-options=s' => \$ssh_options,
   );

my $starting_cwd = cwd();
fatal "cwd() failed: $!" if $starting_cwd eq '';

my $sourcedir = $ARGV[0];

if ($sourcedir eq '') {
   diag "sourcedir missing";
   usage;
}

# read the registration file and lock it since we are going to
# change it.
my $r = read_and_lock_registrations;

# The directory the user wants to unregister may no longer exist,
# so don't require that we be able to cd into it.  However if the
# directory specified on the command line doesn't match anything
# in the registration file, cd to $sourcedir, canonicalize the
# name and see if that matches.
if (! defined($r->{$sourcedir}) && chdir $sourcedir) {
   $sourcedir = cwd();
   fatal "cwd() failed: $!" if $sourcedir eq '';
   chdir $starting_cwd ||
      fatal "couldn't cd back to $starting_cwd: $!";
}

fatal "$sourcedir: directory not registered" if ! defined $r->{$sourcedir};

my ($id, $user, $remotehost, $remotedir) =
  ($r->{$sourcedir}{id},
   $r->{$sourcedir}{user},
   $r->{$sourcedir}{remotehost},
   $r->{$sourcedir}{repositories});

# Launch snap-backend which will do the actual removal of the
# snapshot repository.
if ($remotehost ne '') {
   open2(\*FROMBACKEND, \*TOBACKEND,
         "$ssh_program $ssh_options -x -l $user $remotehost $snap_backend_program");
} else {
   open2(\*FROMBACKEND, \*TOBACKEND, "$snap_backend_program");
}

autoflush TOBACKEND 1;

my $line = read_response;
fatal "no greeting from remote server" if $line eq '';
fatal "remote server greeted us with ", safety_decode($line) if $line !~ /^OK/;

# Send the UNREGISTER command to the backend, which will remove
# the snapshot repository and all snapshots for $sourcedir.
send_command "UNREGISTER", $remotedir, $id;
$line = read_response;
fatal "UNREGISTER failed, remote server said ", safety_decode($line)
   if $line !~ /^OK/;

# Now remove the registration.
# This deletes the $sourcedir key from the hash, NOT the directory
# named by $sourcedir!
delete $r->{$sourcedir};

# write out the remaining registrations
write_registrations($r);

send_command "QUIT";
read_response;

exit 0;
